@@ -229,6 +229,7 @@
 ("content tag" "Inhaltliche Auszeichnung")
 ("context dependent icons" "Kontextabhängige Symbole")
 ("continuation of" "Fortsetzung von")
+("control flow" "Ablaufsteuerung")
 ("convention" "Konvention")
 ("converters" "Konvertierer")
 ("converters to other data formats" "Konvertierung in andere Formate")
@@ -395,7 +396,6 @@
 ("floating figure" "bewegliche Abbildung")
 ("floating object" "bewegliches Objekt")
 ("floating table" "bewegliche Tabelle")
-("flow control" "Ablaufsteuerung")
 ("fold" "zusammenfalten")
 ("fold input field" "Eingabe-Feld zusammenfalten")
 ("folded" "zusammengefaltet")
@@ -413,6 +413,7 @@
 ("footer" "Fußzeile")
 ("footnote" "Fußnote")
 ("for" "für")
+("for all" "für alle")
 ("for each" "für jedes")
 ("foreground" "Vordergrund")
 ("foreign" "andere Alphabete")
@@ -496,6 +497,7 @@
 ("hyphenation" "Trennung")
 ("icelandic" "isländisch")
 ("if" "wenn")
+("iff" "gdw")
 ("image" "Bild")
 ("image links" "Bildverknüpfungen")
 ("implementation" "Implementation")
@@ -1287,7 +1289,7 @@
 ("the primitive TeXmacs constructs" "der grundlegende TeXmacs-Aufbau")
 ("the standard TeXmacs styles" "die Standardstile von TeXmacs")
 ("theorem" "Satz")
-("there are unsaved files" "nicht alle Dateien sind gesichert")
+("there are unsaved documents" "nicht alle Dateien sind gesichert")
 ("this document has been produced using" "Dieses Dokument wurde erzeugt mit")
 ("this line" "diese Zeile")
 ("this page" "diese Seite")
