@@ -1,4 +1,4 @@
-<TeXmacs|1.99.8>
+<TeXmacs|1.99.11>
 
 <style|<tuple|source|std>>
 
@@ -36,6 +36,8 @@
   document has been produced using> GNU <TeXmacs> (<localize|see>
   <with|font-family|tt|http://www.texmacs.org>).<right-flush>>>>>
 
+  <assign|tm-made|<macro|<with|ornament-color||ornament-shadow-color|<value|color>|ornament-sunny-color|<value|color>|ornament-border|1ln|ornament-shape|rounded|ornament-hpadding|0.5spc|ornament-vpadding|0.5spc|<ornament|<math|\<Backsigma\>>>>>>
+
   <assign|TeX|<macro|<active*|T<rsub|<space|-0.4spc><move|<resize|<with|math-level|0|E>||||0.5fn>|0fn|-0.1fn>><space|-0.4spc>X>>>
 
   <assign|LaTeX|<macro|<active*|L<rsup|<space|-0.8spc><move|A|0fn|-0.1fn>><space|-0.2spc>T<rsub|<space|-0.4spc><move|<resize|<with|math-level|0|E>||||0.5fn>|0fn|-0.1fn>><space|-0.4spc>X>>>
@@ -428,7 +430,7 @@
 
   <assign|stressed-distance|0.2fn>
 
-  <assign|stressed|<macro|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<with|color|<value|stressed-color>|<repeat|<with|color|<unquote|<value|color>>|<arg|x>>|<move|<resize|-|<plus|0.6667l|0.3333r>||<plus|0.3333l|0.6667r>|>||<minus|-0.5ex|<value|stressed-distance>>>>>>|<arg|body>>>>>>
+  <assign|stressed|<macro|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<with|color|<value|stressed-color>|<repeat*|<with|color|<unquote|<value|color>>|<arg|x>>|<move|<resize|-|<plus|0.6667l|0.3333r>||<plus|0.3333l|0.6667r>|>||<minus|-0.5ex|<value|stressed-distance>>>>>>|<arg|body>>>>>>
 
   <drd-props|stressed|with-like|yes|arity|1|accessible|all>
 
@@ -526,10 +528,20 @@
     </src-comment>
   </active*>
 
-  <assign|html-div|<macro|name|body|<arg|body>>>
+  <assign|web-title|<macro|title|>>
+
+  <assign|html-tag|<macro|name|body|<arg|body>>>
+
+  <assign|html-attr|<macro|attr|val|body|<arg|body>>>
+
+  <assign|html-div-style|<macro|name|body|<arg|body>>>
+
+  <assign|html-div-class|<macro|name|body|<arg|body>>>
 
   <assign|html-style|<macro|style|body|<arg|body>>>
 
+  <assign|html-class|<macro|style|body|<arg|body>>>
+
   <assign|html-javascript|<macro|script|<small|<colored-frame|pastel
   yellow|<with|font-family|ss|Javascript:
   ><with|font-family|tt|<arg|script>>>>>>
@@ -600,6 +612,24 @@
 
   <assign|gnawed|<macro|body|<add-font-effect|gnawed|<merge|<value|gnawed-strength>|;|<value|gnawed-frequency>>|<arg|body>>>>
 
+  <\active*>
+    <\src-comment>
+      Artistic font effects.
+    </src-comment>
+  </active*>
+
+  <assign|blurred-radius|0.6pt>
+
+  <assign|shadow-font-color|#222>
+
+  <assign|sunny-font-color|#ddd>
+
+  \;
+
+  <assign|blurred|<macro|body|<add-font-effect|blurred|<value|blurred-radius>|<arg|body>>>>
+
+  <assign|enhanced|<macro|body|<add-font-effect|enhanced|<merge|<value|blurred-radius>|;|<value|shadow-font-color>|;|<value|sunny-font-color>>|<arg|body>>>>
+
   <\active*>
     <\src-comment>
       Miscellaneous.
@@ -621,6 +651,10 @@
   <assign|indent-par-first|1.5fn>
 
   <assign|padded-par-par-sep|0.6666fn>
+
+  <assign|math-color|<value|color>>
+
+  <assign|strong-color|<value|color>>
 </body>
 
 <\initial>
