@@ -28,17 +28,21 @@
 
 (define tmhtml-env (make-ahash-table))
 (define tmhtml-css? #t)
+(define tmhtml-mathjax? #f)
 (define tmhtml-mathml? #f)
 (define tmhtml-images? #f)
 (define tmhtml-image-serial 0)
 (define tmhtml-image-cache (make-ahash-table))
 (define tmhtml-image-root-url (unix->url "image"))
 (define tmhtml-image-root-string "image")
+(define tmhtml-site-version #f)
 
 (tm-define (tmhtml-initialize opts)
   (set! tmhtml-env (make-ahash-table))
   (set! tmhtml-css?
 	(== (assoc-ref opts "texmacs->html:css") "on"))
+  (set! tmhtml-mathjax?
+	(== (assoc-ref opts "texmacs->html:mathjax") "on"))
   (set! tmhtml-mathml?
 	(== (assoc-ref opts "texmacs->html:mathml") "on"))
   (set! tmhtml-images?
@@ -84,6 +88,8 @@
 	((string-starts? s "<frak-")
 	 `(h:u ,(tmhtml-sub-token s 6)))
 	((string-starts? s "<bbb-") `(h:u (h:b ,(tmhtml-sub-token s 5))))
+	((string-starts? s "<up-") (tmhtml-sub-token s 4))
+	((string-starts? s "<b-up-") `(h:b ,(tmhtml-sub-token s 6)))
 	((string-starts? s "<b-") `(h:b (h:var ,(tmhtml-sub-token s 3))))
 	((string-starts? s "<")
 	 (with encoded (cork->utf8 s)
@@ -120,6 +126,7 @@
 (define (tmhtml-find-title doc)
   (cond ((npair? doc) #f)
 	((func? doc 'doc-title 1) (cadr doc))
+	((func? doc 'web-title 1) (cadr doc))
 	((func? doc 'tmdoc-title 1) (cadr doc))
 	((func? doc 'tmdoc-title* 2) (cadr doc))
 	((func? doc 'tmdoc-title** 3) (caddr doc))
@@ -137,6 +144,9 @@
 	  "h6 { display: inline; padding-right: 1em } "
 	  "table { border-collapse: collapse } "
 	  "td { padding: 0.2em; vertical-align: baseline } "
+          "dt { float: left; min-width: 1.75em; text-align: right; "
+          "padding-right: 0.75em; font-weight: bold; } "
+          "dd { margin-left: 2.5em; } "          
 	  ".subsup { display: inline; vertical-align: -0.2em } "
 	  ".subsup td { padding: 0px; text-align: left} "
 	  ".fraction { display: inline; vertical-align: -0.8em } "
@@ -148,30 +158,37 @@
 	  ".compact-block p { margin-top: 0px; margin-bottom: 0px } "
 	  ".left-tab { text-align: left } "
 	  ".center-tab { text-align: center } "
-          ".balloon-anchor { border-bottom: 1px dotted #000000; outline:none;"
-          "                  cursor: help; position: relative; }"
+          ".balloon-anchor { border-bottom: 1px dotted #000000; outline: none;"
+          " cursor: help; position: relative; } "
           ".balloon-anchor [hidden] { margin-left: -999em; position: absolute;"
-                                    " display: none; }"
-          ".balloon-anchor:hover [hidden] { position: absolute; left: 1em;"
-                                 " top: 2em; z-index: 99; margin-left: 0;"
-                                 " width: 500px; display: inline-block; }"
-          ".balloon-body { }"
-	  ".ornament  { border-width: 1px; border-style: solid; border-color: "
-                      " black; display: inline-block; padding: 0.2em; } "
+	  " display: none; } "
+          ".balloon-anchor: hover [hidden] { position: absolute; left: 1em;"
+	  " top: 2em; z-index: 99; margin-left: 0;"
+	  " width: 500px; display: inline-block; } "
+          ".balloon-body { } "
+	  ".ornament { border-width: 1px; border-style: solid;"
+	  " border-color: black; display: inline-block; padding: 0.2em; } "
 	  ".right-tab { float: right; position: relative; top: -1em } "))
 	(mathml "math { font-family: cmr, times, verdana } "))
     (if tmhtml-mathml? (string-append html mathml) html)))
 
-(define (with-extract w var)
+(define (with-extract-sub w var post)
   (cond ((and (pair? w) (== (car w) 'with)
 	      (pair? (cdr w)) (== (cadr w) var)
 	      (pair? (cddr w)))
-	 (tmhtml-force-string (caddr w)))
+	 (post (caddr w)))
 	((and (pair? w) (== (car w) 'with)
 	      (pair? (cdr w)) (pair? (cddr w)))
-	 (with-extract `(with ,@(cdddr w)) var))
+	 (with-extract-sub `(with ,@(cdddr w)) var post))
 	(else #f)))
 
+(define (with-extract w var)
+  (with-extract-sub w var tmhtml-force-string))
+
+(define (with-extract* w var)
+  (with post (lambda (x) (if (tm-func? x 'quote 1) (cadr x) x))
+    (with-extract-sub w var post)))
+
 (define (tmhtml-file l)
   ;; This handler is special:
   ;; Since !file is a special node used only at the top of trees
@@ -184,11 +201,16 @@
 	 (css `(h:style (@ (type "text/css")) ,(tmhtml-css-header)))
 	 (xhead '())
 	 (body (tmhtml doc)))
+    (set! tmhtml-site-version
+          (with-extract doc "html-site-version"))
     (set! title
 	  (cond ((with-extract doc "html-title")
-		 (with-extract doc "html-title"))
+                 (with-extract doc "html-title"))
+		((and (not title) (with-extract doc "html-doc-title"))
+                 (with-extract doc "html-doc-title"))
 		((not title) "No title")
-		((or (in? "tmdoc" styles) (in? "tmweb" styles))
+		((or (in? "tmdoc" styles)
+                     (in? "tmweb" styles) (in? "tmweb2" styles))
 		 `(concat ,(tmhtml-force-string title)
 			  " (FSF GNU project)"))
 		(else (tmhtml-force-string title))))
@@ -200,13 +222,40 @@
 		(else css)))
     (if (with-extract doc "html-head-javascript-src")
 	(let* ((src (with-extract doc "html-head-javascript-src"))
-	       (script `(h:script (@ (language "javascript") (src ,src)))))
+	       (script `(h:script (@ (language "javascript")
+                                     (src ,src)
+                                     (defer "<implicit>")))))
 	  (set! xhead (append xhead (list script)))))
     (if (with-extract doc "html-head-javascript")
 	(let* ((code (with-extract doc "html-head-javascript"))
-	       (script `(h:script (@ (language "javascript")) ,code)))
+	       (script `(h:script (@ (language "javascript")
+                                     (defer "<implicit>")) ,code)))
 	  (set! xhead (append xhead (list script)))))
-    (if (or (in? "tmdoc" styles) (in? "tmweb" styles)
+    (if (tm-func? (with-extract* doc "html-extra-css") 'tuple)
+        (for (src (cdr (with-extract* doc "html-extra-css")))
+          (with link-css `(h:link (@ (rel "stylesheet")
+                                     (href ,src)
+                                     (type "text/css")))
+            (set! xhead (append xhead (list link-css))))))
+    (if (tm-func? (with-extract* doc "html-extra-javascript-src") 'tuple)
+        (for (src (cdr (with-extract* doc "html-extra-javascript-src")))
+          (with script `(h:script (@ (language "javascript")
+                                     (src ,src)
+                                     (defer "<implicit>")))
+            (set! xhead (append xhead (list script))))))
+    (if (tm-func? (with-extract* doc "html-extra-javascript") 'tuple)
+        (for (code (cdr (with-extract* doc "html-extra-javascript")))
+          (with script `(h:script (@ (language "javascript")
+                                     (defer "<implicit>")) ,code)
+            (set! xhead (append xhead (list script))))))
+    (if tmhtml-mathjax?
+	(let* ((site "http://cdn.mathjax.org/")
+               (loc "mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML")
+               (src (string-append site loc))
+	       (script `(h:script (@ (language "javascript") (src ,src)))))
+	  (set! xhead (append xhead (list script)))))
+    (if (or (in? "tmdoc" styles)
+            (in? "tmweb" styles) (in? "tmweb2" styles)
             (in? "mmxdoc" styles) (in? "magix-web" styles)
             (in? "max-web" styles))
 	(set! body (tmhtml-tmdoc-post body)))
@@ -394,6 +443,10 @@
 	 (serialize-concat (cadr x))
 	 (serialize-concat (cadddr x))
 	 (serialize-concat (caddr x)))
+        ((func? x 'with 1) (serialize-concat (cadr x)))
+        ((and (func? x 'with)
+              (in? cadr (list "locus-color" "visited-color")))
+         (serialize-concat `(with ,@(cdddr x))))
 	((func? x 'with)
 	 (let* ((r (simplify-document (cAr x)))
 		(w (lambda (y) `(with ,@(cDdr x) ,y))))
@@ -405,6 +458,7 @@
 		 (serialize-paragraph (w head))
 		 (set! document-done (cons (w body) document-done))
 		 (serialize-concat (w tail))))))
+        ((func? x 'locus) (serialize-concat (cAr x)))
 	(else (serialize-print x))))
 
 (define (simplify-document x)
@@ -544,6 +598,16 @@
 ;; Formatting text
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define (tmhtml-hidden l)
+  ;; FIXME: distinguish inline and block?
+  (with r (tmhtml (car l))
+    `((div (@ (class "toggle") (style "display: none")) ,@r))))
+
+(define (tmhtml-shown l)
+  ;; FIXME: distinguish inline and block?
+  (with r (tmhtml (car l))
+    `((div (@ (class "toggle") (style "display: block")) ,@r))))
+
 (define (tmhtml-hspace l)
   (with len (tmlength->htmllength (if (list-1? l) (car l) (cadr l)) #t)
     (if (not len) '()
@@ -751,17 +815,24 @@
   (ahash-with tmhtml-env :math (== val "math")
     (tmhtml (if (== val "prog") `(verbatim ,arg) arg))))
 
+(define (tmhtml-with-math-display val arg)
+  (ahash-with tmhtml-env :math-display (== val "true")
+    (tmhtml arg)))
+
 (define (tmhtml-with-color val arg)
   `((h:font (@ (color ,(tmcolor->htmlcolor val))) ,@(tmhtml arg))))
 
 (define (tmhtml-with-font-size val arg)
   (ahash-with tmhtml-env :mag val
     (let* ((x (* (string->number val) 100))
+           (c (string-append "font-size: " (number->string x) "%"))
 	   (s (cond ((< x 1) "-4") ((< x 55) "-4") ((< x 65) "-3")
 		    ((< x 75) "-2") ((< x 95) "-1") ((< x 115) "0")
 		    ((< x 135) "+1") ((< x 155) "+2") ((< x 185) "+3")
 		    ((< x 225) "+4") ((< x 500) "+5") (else "+5"))))
-      (if s `((h:font (@ (size ,s)) ,@(tmhtml arg))) (tmhtml arg)))))
+      (cond (tmhtml-css? `((h:font (@ (style ,c)) ,@(tmhtml arg))))
+            (s `((h:font (@ (size ,s)) ,@(tmhtml arg))))
+            (else (tmhtml arg))))))
 
 (define (tmhtml-with-block style arg)
   (with r (tmhtml (blockify arg))
@@ -841,6 +912,14 @@
   ;; Explicit expansions are converted and handled as implicit expansions.
   (tmhtml (cadr l)))
 
+(define (tmhtml-include l)
+  (if (or (npair? l) (nstring? (car l))) '()
+      (with u (url-relative current-save-source (unix->url (car l)))
+	(if (not (url-exists? u)) '()
+	    (with-global current-save-source u
+	      (with t (tree-inclusion u)
+		(tmhtml (tm->stree t))))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Source code
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -894,7 +973,9 @@
   ;; in order to disable this suffix change
   (let* ((sdir (string-rindex s #\/))
 	 (sep (string-rindex s #\#)))
-    (cond ((or (string-starts? s "http:") (string-starts? s "ftp:")) s)
+    (cond ((or (string-starts? s "http:")
+               (string-starts? s "https:")
+               (string-starts? s "ftp:")) s)
           ((and sep (or (not sdir) (< sdir sep)))
 	   (string-append (tmhtml-suffix (substring s 0 sep))
 			  (string-drop s sep)))
@@ -922,7 +1003,9 @@
 	(else '())))
 
 (define (tmhtml-action l)
-  `((h:u ,@(tmhtml (car l)))))
+  (if tmhtml-css?
+      (tmhtml (car l))
+      `((h:u ,@(tmhtml (car l))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Tables
@@ -1046,10 +1129,9 @@
 
 (define (tmhtml-make-table t tablef colf rowf cellf)
   (let* ((attrs (map* tmhtml-make-table-attr tablef))
-	 (em (- (* (tmtable-rows t) 0.55)))
-	 (va (string-append "vertical-align: " (number->htmlstring em) "em")))
+         (va "vertical-align: middle"))
     (if (not (list-find attrs (cut == <> "width: 100%")))
-	(set! attrs (cons* "display: inline" va attrs)))
+	(set! attrs (cons* "display: inline-table" va attrs)))
     `(h:table ,@(html-css-attrs attrs)
 	      ,@(tmhtml-make-column-group colf)
 	      (h:tbody ,@(tmhtml-make-rows (cdr t) rowf cellf)))))
@@ -1081,28 +1163,58 @@
 
 (define (tmhtml-png y)
   (let* ((mag (ahash-ref tmhtml-env :mag))
-	 (x (if (or (nstring? mag) (== mag "1")) y
-		`(with "font-size" ,mag ,y)))
+	 (x (if (or tmhtml-css? (nstring? mag) (== mag "1")) y
+                (with nmag `(times (value "magnification") ,mag)
+                  `(with "magnification" ,nmag ,y))))
 	 (l1 (tmhtml-collect-labels y))
 	 (l2 (if (null? l1) l1 (list (car l1)))))
     (with cached (ahash-ref tmhtml-image-cache x)
       (if (not cached)
 	  (receive (name-url name-string) (tmhtml-image-names "png")
 	    ;;(display* x " -> " name-url ", " name-string "\n")
-	    (let* ((extents (print-snippet name-url x))
-		   ;;(pixels (inexact->exact (/ (second extents) 2100)))
-		   (pixels (inexact->exact (/ (second extents) 2000)))
-		   (valign (number->htmlstring pixels))
-		   (style (string-append "vertical-align: " valign "px")))
+	    (let* ((extents (print-snippet name-url x #t))
+                   (unit (/ 6000.0
+                            (* (ninth extents) (tenth extents) 20625.0)))
+		   (x3 (* (first extents) unit))
+		   (y3 (* (second extents) unit))
+		   (x4 (* (third extents) unit))
+		   (y4 (* (fourth extents) unit))
+		   (x1 (* (fifth extents) unit))
+		   (y1 (* (sixth extents) unit))
+		   (x2 (* (seventh extents) unit))
+		   (y2 (* (eighth extents) unit))
+                   (lmar (number->htmlstring (- x3 x1)))
+                   (bmar (number->htmlstring (- y3 y1)))
+                   (rmar (number->htmlstring (- x2 x4)))
+                   (tmar (number->htmlstring (- y2 y4)))
+		   (valign (number->htmlstring (- y3 (- y3 y1))))
+		   (height (number->htmlstring (- y4 y3)))
+		   (style (string-append "margin-left: " lmar "em; "
+                                         "margin-bottom: " bmar "em; "
+                                         "margin-right: " rmar "em; "
+                                         "margin-top: " tmar "em; "
+                                         "vertical-align: " valign "em; "
+                                         "height: " height "em"))
+                   (attrs (if tmhtml-css?
+                              `((src ,name-string) (style ,style) ,@l2)
+                              `((src ,name-string) ,@l2)))
+                   (img (if (url-exists? name-url)
+                            `((h:img (@ ,@attrs)))
+                            `())))
 	      ;;(display* x " -> " extents "\n")
-	      (set! cached
-		    `((h:img (@ (src ,name-string) (style ,style) ,@l2))))
+	      (set! cached img)
 	      (ahash-set! tmhtml-image-cache x cached)))
 	  cached))))
 
 (define (tmhtml-graphics l)
   (tmhtml-png (cons 'graphics l)))
 
+(define (tmhtml-draw-over l)
+  (tmhtml-png (cons 'draw-over l)))
+
+(define (tmhtml-draw-under l)
+  (tmhtml-png (cons 'draw-under l)))
+
 (define (tmhtml-image-name name)
   ;; FIXME: we should replace ~, environment variables, etc.
   (with u (url-relative current-save-target (unix->url name))
@@ -1255,11 +1367,19 @@
   ;; LaTeX it is used to escape special characters (and protect multiple inline
   ;; spaces, yuck!), but in TeXmacs there is no such problem.
   (with body (first args)
-    (if (stm-block-structure? body)
-	(verbatim-pre
-	 (ahash-with tmhtml-env :preformatted #t
-		     (tmhtml body)))
-	(verbatim-tt (tmhtml body)))))
+    (with pre? #f
+      (when (func? body 'with 1)
+      (set! body (cAr body)))
+      (when (and (func? body 'with 3) (== (cadr body) "locus-color"))
+        (set! body (cAr body)))
+      (when (func? body 'action)
+        (set! body (cadr body))
+        (set! pre? (string? body)))
+      (if (or (stm-block-structure? body) pre?)
+          (verbatim-pre
+           (ahash-with tmhtml-env :preformatted #t
+             (tmhtml body)))
+          (verbatim-tt (tmhtml body))))))
 
 (define (verbatim-tt content)
   `((h:tt (@ (class "verbatim")) ,@content)))
@@ -1278,29 +1398,94 @@
 (define (tmhtml-equation* l)
   (with first (simplify-document (car l))
     (with x `(with "mode" "math" (with "math-display" "true" ,first))
-      `((h:center ,@(tmhtml x))))))
+      (ahash-with tmhtml-env :math-display #t
+        `((h:center ,@(tmhtml x)))))))
 
 (define (tmhtml-equation-lab l)
   (with first (simplify-document (car l))
     (with x `(with "mode" "math" (with "math-display" "true" ,first))
-      `((h:table (@ (width "100%"))
-		 (h:tr (h:td (@ (align "center") (width "100%"))
-			     ,@(tmhtml x))
-		       (h:td (@ (align "right"))
-			     "(" ,@(tmhtml (cadr l)) ")")))))))
+      (ahash-with tmhtml-env :math-display #t
+        `((h:table (@ (width "100%"))
+                   (h:tr (h:td (@ (align "center") (width "100%"))
+                               ,@(tmhtml x))
+                         (h:td (@ (align "right"))
+                               "(" ,@(tmhtml (cadr l)) ")"))))))))
+
+(define (tmhtml-mathjax-formula* x)
+  ;;(display* "x= " x "\n")
+  (let* ((opts (list (cons "texmacs->latex:mathjax" "on")))
+         (s (serialize-latex (texmacs->latex x opts)))
+         (display? (ahash-ref tmhtml-env :math-display))
+         (style (if display? "\\displaystyle " ""))
+         (mj (string-append "\\(" style s "\\)")))
+    (when (and (string-starts? mj "\\(\\displaystyle \\begin{array}{rcl}")
+               (string-ends?   mj "\\end{array}\\)"))
+      (set! mj (string-append "\\begin{eqnarray*}"
+                              (substring mj 34 (- (string-length mj) 13))
+                              "\\end{eqnarray*}")))
+    (list mj)))
+
+(define (tmhtml-mathjax-formula x)
+  (cond ((tm-func? x 'with 1)
+         (tmhtml-mathjax-formula (cadr x)))
+        ((and (tm-func? x 'with 3)
+              (== (cadr x) "math-display")
+              (== (caddr x) "true"))
+         (ahash-with tmhtml-env :math-display #t
+           (tmhtml-mathjax-formula (cadddr x))))
+        (else (tmhtml-mathjax-formula* x))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Tags for customized html generation
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (tmhtml-html-div l)
-  (list `(h:div (@ (class ,(tmhtml-force-string (car l))))
-		,@(tmhtml (cadr l)))))
+(define (tmhtml-append-attribute-sub l var val)
+  (cond ((null? l) (list (list var val)))
+	((and (list-2? (car l)) (== (caar l) var) (string? (cadar l)))
+	 (cons (list var (string-append (cadar l) " " val)) (cdr l)))
+	(else (cons (car l) (tmhtml-append-attribute-sub (cdr l) var val)))))
+
+(define (tmhtml-append-attribute t var val)
+  (cond ((and (func? t 'h:p 1) (func? (cadr t) 'h:p))
+         (tmhtml-append-attribute (cadr t) var val))
+        ((and (pair? t) (pair? (cdr t)) (list? t)
+              (pair? (cadr t)) (== (caadr t) '@) (list? (cadr t)))
+         (with l (tmhtml-append-attribute-sub (cdadr t) var val)
+           `(,(car t) (@ ,@l) ,@(cddr t))))
+        ((and (pair? t) (list? t))
+         `(,(car t) (@ (,var ,val)) ,@(cdr t)))
+        ((== var 'class) `(font (@ (,var ,val)) ,t))
+        (else `(class (@ (,var ,val)) ,t))))
+
+(define (tmhtml-html-tag l)
+  (let* ((s (tmhtml-force-string (car l)))
+         (r (tmhtml (cadr l))))
+    (list `(,(string->symbol s) ,@r))))
+
+(define (tmhtml-html-attr l)
+  (let* ((a (string->symbol (tmhtml-force-string (car l))))
+         (v (tmhtml-force-string (cadr l)))
+         (r (tmhtml (caddr l))))
+    (map (cut tmhtml-append-attribute <> a v) r)))
 
 (define (tmhtml-html-style l)
+  (let* ((s (tmhtml-force-string (car l)))
+         (r (tmhtml (cadr l))))
+    (map (cut tmhtml-append-attribute <> 'style s) r)))
+
+(define (tmhtml-html-class l)
+  (let* ((s (tmhtml-force-string (car l)))
+         (r (tmhtml (cadr l))))
+    (map (cut tmhtml-append-attribute <> 'class s) r)))
+
+(define (tmhtml-html-div-style l)
   (list `(h:div (@ (style ,(tmhtml-force-string (car l))))
 		,@(tmhtml (cadr l)))))
 
+(define (tmhtml-html-div-class l)
+  (list `(h:div (@ (class ,(tmhtml-force-string (car l))))
+		,@(tmhtml (cadr l)))))
+
 (define (tmhtml-html-javascript l)
   (list `(h:script (@ (language "javascript"))
 		   ,(tmhtml-force-string (car l)))))
@@ -1415,25 +1600,31 @@
 
 (define (tmhtml-implicit-compound l)
   (or (tmhtml-dispatch 'tmhtml-stdmarkup% l)
+      (and (!= tmhtml-site-version "2")
+           (tmhtml-dispatch 'tmhtml-tmdocmarkup% l))
       '()))
 
 (tm-define (tmhtml-root x)
   (ahash-with tmhtml-env :mag "1"
     (ahash-with tmhtml-env :math #f
-      (ahash-with tmhtml-env :preformatted #f
-	(ahash-with tmhtml-env :left-margin 0
-	  (ahash-with tmhtml-env :right-margin 0
-	    (tmhtml x)))))))
+      (ahash-with tmhtml-env :math-display #f
+        (ahash-with tmhtml-env :preformatted #f
+          (ahash-with tmhtml-env :left-margin 0
+            (ahash-with tmhtml-env :right-margin 0
+              (tmhtml x))))))))
 
 (define (tmhtml x)
   ;; Main conversion function.
   ;; Takes a TeXmacs tree in Scheme notation and produce a SXML node-set.
   ;; All handler functions have a similar prototype.
-  (cond ((and tmhtml-mathml? (ahash-ref tmhtml-env :math))
+  (cond ((and tmhtml-mathjax? (ahash-ref tmhtml-env :math))
+         (tmhtml-mathjax-formula x))
+        ((and tmhtml-mathml? (ahash-ref tmhtml-env :math))
 	 `((m:math (@ (xmlns "http://www.w3.org/1998/Math/MathML"))
 		   ,(texmacs->mathml x tmhtml-env))))
-	((and tmhtml-images? (ahash-ref tmhtml-env :math))
-	 (tmhtml-png `(with "mode" "math" ,x)))
+	((and tmhtml-images? (ahash-ref tmhtml-env :math)
+              (!= tmhtml-image-root-string "image"))
+         (tmhtml-png `(with "mode" "math" ,x)))
 	((string? x)
 	 (if (string-null? x) '() (tmhtml-text x))) ; non-verbatim string nodes
 	(else (or (tmhtml-dispatch 'tmhtml-primitives% x)
@@ -1449,7 +1640,10 @@
   (surround tmhtml-surround)
   (concat tmhtml-concat)
   (rigid tmhtml-id)
-  (format tmhtml-noop)
+  (hgroup tmhtml-id)
+  (hidden tmhtml-hidden)
+  (freeze tmhtml-id)
+  (unfreeze tmhtml-id)
   (hspace tmhtml-hspace)
   (vspace* tmhtml-vspace)
   (vspace tmhtml-vspace)
@@ -1457,15 +1651,17 @@
   (htab tmhtml-hspace)
   (split tmhtml-noop)
   (move tmhtml-move)
+  (shift tmhtml-move)
   (resize tmhtml-resize)
-  (float tmhtml-float)
+  (clipped tmhtml-resize)
   (repeat tmhtml-repeat)
+  (repeat* tmhtml-repeat)
+  (float tmhtml-float)
   (datoms tmhtml-datoms)
   (dlines tmhtml-datoms)
   (dpages tmhtml-datoms)
   (dbox tmhtml-datoms)
-  (locus tmhtml-datoms)
-
+  
   (with-limits tmhtml-noop)
   (line-break tmhtml-noop)
   (new-line tmhtml-new-line)
@@ -1480,6 +1676,10 @@
   (page-break tmhtml-noop)
   (no-page-break* tmhtml-noop)
   (no-page-break tmhtml-noop)
+  (no-break-here* tmhtml-noop)
+  (no-break-here tmhtml-noop)
+  (no-break-start tmhtml-noop)
+  (no-break-end tmhtml-noop)
   (new-page* tmhtml-noop)
   (new-page tmhtml-noop)
   (new-dpage* tmhtml-noop)
@@ -1518,7 +1718,9 @@
   ((:or xmacro get-label get-arity map-args eval-args) tmhtml-noop)
   (mark tmhtml-mark)
   (eval tmhtml-noop)
-  ((:or if if* case while for-each extern include use-package) tmhtml-noop)
+  ((:or if if* case while for-each extern) tmhtml-noop)
+  (include tmhtml-include)
+  (use-package tmhtml-noop)
 
   ((:or or xor and not plus minus times over div mod merge length range
 	number date translate is-tuple look-up equal unequal less lesseq
@@ -1535,22 +1737,30 @@
   (latex tmhtml-noop)
   (hybrid tmhtml-noop)
 
-  ((:or tuple collection associate) tmhtml-noop)
+  (locus tmhtml-datoms)
   (label tmhtml-label)
   (reference tmhtml-noop)
   (pageref tmhtml-noop)
-  (write tmhtml-noop)
-  (specific tmhtml-specific)
   (hlink tmhtml-hyperlink)
   (action tmhtml-action)
-  ((:or tag meaning) tmhtml-noop)
-  ((:or switch fold exclusive progressive superposed) tmhtml-noop)
+  (write tmhtml-noop)
+  
+  ((:or tuple collection associate) tmhtml-noop)
+  (specific tmhtml-specific)
+
   (graphics tmhtml-graphics)
   ((:or point line arc bezier) tmhtml-noop)
   (image tmhtml-image)
-  (ornament tmhtml-ornament)
-  ((:or mouse-over-balloon mouse-over-balloon*) tmhtml-balloon)
 
+  (ornament tmhtml-ornament)
+
+  (format tmhtml-noop)
+  ((:or tag meaning) tmhtml-noop)
+
+  (vgroup tmhtml-id)
+  ((:or switch fold exclusive progressive superposed) tmhtml-noop)
+  ((:or mouse-over-balloon mouse-over-balloon*) tmhtml-balloon)
+  
   (!file tmhtml-file))
 
 (logic-table tmhtml-stdmarkup%
@@ -1572,7 +1782,7 @@
 	enumerate-alpha enumerate-Alpha)
    ,tmhtml-enumerate)
   ((:or description description-compact description-dash
-	description-align description-long)
+	description-align description-long description-paragraphs)
    ,tmhtml-description)
   (item* (h:dt))
   (!item ,tmhtml-post-item)
@@ -1597,32 +1807,44 @@
   (TeX ,(lambda x '("TeX")))
   (LaTeX ,(lambda x '("LaTeX")))
   ;; additional tags
+  (shown ,tmhtml-shown)
   (hidden-title ,tmhtml-noop)
   (doc-title-block ,tmhtml-doc-title-block)
   (equation* ,tmhtml-equation*)
   (equation-lab ,tmhtml-equation-lab)
   (equations-base ,tmhtml-equation*)
+  (wide-float ,tmhtml-float)
+  (draw-over ,tmhtml-draw-over)
+  (draw-under ,tmhtml-draw-under)
   ;; tags for customized html generation
-  (html-div ,tmhtml-html-div)
+  (html-tag ,tmhtml-html-tag)
+  (html-attr ,tmhtml-html-attr)
+  (html-div-style ,tmhtml-html-div-style)
+  (html-div-class ,tmhtml-html-div-class)
   (html-style ,tmhtml-html-style)
+  (html-class ,tmhtml-html-class)
   (html-javascript ,tmhtml-html-javascript)
   (html-javascript-src ,tmhtml-html-javascript-src)
   (html-video ,tmhtml-html-video)
   ;; tmdoc tags
+  (web-title ,tmhtml-noop)
+  (hyper-link ,tmhtml-hyperlink))
+
+(logic-table tmhtml-tmdocmarkup%
+  ;; deprecated direct exports of tmdoc tags
   (tmdoc-title ,tmhtml-tmdoc-title)
   (tmdoc-title* ,tmhtml-tmdoc-title*)
   (tmdoc-title** ,tmhtml-tmdoc-title**)
   (tmdoc-flag ,tmhtml-tmdoc-flag)
   (tmdoc-copyright ,tmhtml-tmdoc-copyright)
-  (tmdoc-license ,tmhtml-tmdoc-license)
-  (key ,tmhtml-key)
-  (hyper-link ,tmhtml-hyperlink))
+  (tmdoc-license ,tmhtml-tmdoc-license))
 
-;;    (name (h:name)) ; not in HTML4
-;;    (person (h:person)))) ; not in HTML4
+;; (name (h:name)) ; not in HTML4
+;; (person (h:person)))) ; not in HTML4
 
 (logic-table tmhtml-with-cmd%
   ("mode" ,tmhtml-with-mode)
+  ("math-display" ,tmhtml-with-math-display)
   ("color" ,tmhtml-with-color)
   ("font-size" ,tmhtml-with-font-size)
   ("par-left" ,tmhtml-with-par-left)
