@@ -249,8 +249,17 @@
 (define (make-enum p style)
   "Make @(enum :%3 :string?) item."
   (with (tag cmd vals val width) p
-    (widget-enum (object->command (menu-protect cmd))
-                 (vals) (val) style width)))
+    (let* ((xval (val))
+           (xvals (vals))
+           (nvals (if (and (nnull? xvals) (== (cAr xvals) ""))
+                      `(,@(cDr xvals) ,xval "") `(,@xvals ,xval)))
+           (xvals* (list-remove-duplicates nvals))
+           (tval (translate xval))
+           (tvals (map translate xvals*))
+           (dec (map (lambda (v) (cons (translate v) v)) xvals*))
+           (cmd* (lambda (r) (cmd (or (assoc-ref dec r) r)))))
+      (widget-enum (object->command (menu-protect cmd*))
+                   tvals tval style width))))
 
 (define (make-choice p style)
   "Make @(choice :%3) item."
@@ -988,7 +997,7 @@
            (men (menu-promise))
            (scm (list 'vertical men))
            (wid (make-menu-widget scm 0)))
-      (alt-window-create-quit win wid name qui)
+      (alt-window-create-quit win wid (translate name) qui)
       (alt-window-show win))))
 
 (tm-define (dialogue-window menu-promise cmd name . opts)
@@ -1001,7 +1010,7 @@
            (men (menu-promise lbd))
            (scm (list 'vertical men))
            (wid (make-menu-widget scm 0)))
-      (alt-window-create-quit win wid name qui)
+      (alt-window-create-quit win wid (translate name) qui)
       (alt-window-show win))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1014,7 +1023,7 @@
          (lbd (lambda x (apply cmd x) (alt-window-delete win)))
          (com (object->command (menu-protect lbd)))
          (wid (wid-promise com)))
-    (alt-window-create win wid name #t)
+    (alt-window-create win wid (translate name) #t)
     (alt-window-show win)))
 
 (tm-define (interactive-print done u)
@@ -1029,6 +1038,7 @@
 
 (tm-define (interactive-color cmd proposals)
   (:interactive #t)
+  (set! proposals (map tm->tree proposals))
   (if (not (qt-gui?))
       (interactive-rgb-picker cmd proposals)
       (with p (lambda (com) (widget-color-picker com #f proposals))
@@ -1037,6 +1047,7 @@
 
 (tm-define (interactive-background cmd proposals)
   (:interactive #t)
+  (set! proposals (map tm->tree proposals))
   (if (not (qt-gui?))
       (interactive-rgb-picker cmd proposals)
       ;;(with p (lambda (com) (widget-color-picker com #t proposals))
