@@ -18,6 +18,50 @@
         (math math-edit)
         (table table-edit)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Bypassing the pre-edit mechanism
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (disable-pre-edit? key)
+  (:mode in-math?)
+  (in? key (list "^" "~" "`")))
+
+(tm-define (downgrade-pre-edit key)
+  (:mode in-math?)
+  (cond ((in? key (list "à" "á" "â" "ã" "ä")) "a")
+        ((in? key (list "ĉ")) "c")
+        ((in? key (list "è" "é" "ê" "ẽ" "ë")) "e")
+        ((in? key (list "ĝ")) "g")
+        ((in? key (list "ĥ")) "h")
+        ((in? key (list "ì" "í" "î" "ĩ" "ï")) "i")
+        ((in? key (list "ĵ")) "j")
+        ((in? key (list "m̂")) "m")
+        ((in? key (list "ǹ" "ń" "n̂" "ñ")) "n")
+        ((in? key (list "ò" "ó" "ô" "õ" "ö")) "o")
+        ((in? key (list "ŝ")) "s")
+        ((in? key (list "ù" "ú" "û" "ũ" "ü")) "u")
+        ((in? key (list "ṽ")) "v")
+        ((in? key (list "ẁ" "ŵ")) "w")
+        ((in? key (list "ỳ" "ý" "ŷ" "ỹ" "ÿ")) "y")
+        ((in? key (list "ẑ")) "z")
+        ((in? key (list "À" "Á" "Â" "Ã" "Ä")) "A")
+        ((in? key (list "Ĉ")) "C")
+        ((in? key (list "È" "É" "Ê" "Ẽ" "Ë")) "E")
+        ((in? key (list "Ĝ")) "G")
+        ((in? key (list "Ĥ")) "H")
+        ((in? key (list "Ì" "Í" "Î" "Ĩ" "Ï")) "I")
+        ((in? key (list "Ĵ")) "J")
+        ((in? key (list "M̂")) "M")
+        ((in? key (list "Ǹ" "Ń" "N̂" "Ñ")) "N")
+        ((in? key (list "Ò" "Ó" "Ô" "Õ" "Ö")) "O")
+        ((in? key (list "Ŝ")) "S")
+        ((in? key (list "Ù" "Ú" "Û" "Ũ" "Ü")) "U")
+        ((in? key (list "Ṽ")) "V")
+        ((in? key (list "Ẁ" "Ŵ")) "W")
+        ((in? key (list "Ỳ" "Ý" "Ŷ" "Ỹ" "Ÿ")) "Y")
+        ((in? key (list "Ẑ")) "Z")
+        (else "")))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Hacks to work around the problem that MacOS reserves Alt-based shortcuts
 ;; for its own personal use
@@ -103,6 +147,14 @@
   ("math:under accent:check" (make-wide-under "<check>"))
   ;; end
 
+  ("S-F5 S-F5" (make 'math-sl))
+  ("S-F6 S-F6" (make 'math-up))
+  ("F5 F5" (make 'math-it))
+  ("F6 F6" (make 'math-bf))
+  ("F7 F7" (make 'math-tt))
+  ("F8 F8" (make 'math-ss))
+  ("altcmd S-F5" (make 'math-sl))
+  ("altcmd S-F6" (make 'math-up))
   ("altcmd F5" (make 'math-it))
   ("altcmd F6" (make 'math-bf))
   ("altcmd F7" (make 'math-tt))
@@ -899,6 +951,18 @@
   ("< = = >" "<Longleftrightarrow>")
   ("< = = > var" "<Longupdownarrow>")
   ("< = = > . ." "<Longleftrightarrow><cdots><Longleftrightarrow>")
+  ("= var var >" "<Rrightarrow>")
+  ("= > var var var" "<Rrightarrow>")
+  ("< = var var var var" "<Lleftarrow>")
+  ("< = var var var var >" "<LRleftrightarrow>")
+  ("< = > var var" "<LRleftrightarrow>")
+  ("= = var var" "<longequiv>")
+  ("= = var var >" "<Llongrightarrow>")
+  ("= = > var var var" "<Llongrightarrow>")
+  ("< = = var var var" "<Llongleftarrow>")
+  ("< = = var var var >" "<LRleftrightarrow>")
+  ("< = = > var var" "<Llongleftrightarrow>")
+  
   ("- / var" "<rightharpoondown>")
   ("- / var var" "<rightharpoonup>")
   ("- / var var var" "<downharpoonleft>")
@@ -918,18 +982,6 @@
   ("/ - var -" "<longleftharpoonup>")
   ("/ - var - var" "<longleftharpoondown>")
   ("/ - var var -" "<longleftharpoondown>")
-  ("| - >" "<mapsto>")
-  ("| - > var" "<hookrightarrow>")
-  ("| - > var var" "<mapsdown>")
-  ("| - > var var var" "<hookdownarrow>")
-  ("| - > var var var var" "<mapsup>")
-  ("| - > var var var var var" "<hookuparrow>")
-  ("| - - >" "<longmapsto>")
-  ("| - - > var" "<longhookrightarrow>")
-  ("| - - > var var" "<longmapsdown>")
-  ("| - - > var var var" "<longhookdownarrow>")
-  ("| - - > var var var var" "<longmapsup>")
-  ("| - - > var var var var var" "<longhookuparrow>")
   ("| var - >" "<mapsto>")
   ("| var - > var" "<hookrightarrow>")
   ("| var - > var var" "<mapsdown>")
@@ -942,6 +994,18 @@
   ("| var - - > var var var" "<longhookdownarrow>")
   ("| var - - > var var var var" "<longmapsup>")
   ("| var - - > var var var var var" "<longhookuparrow>")
+  ("| - >" "<mapsto>")
+  ("| - > var" "<hookrightarrow>")
+  ("| - > var var" "<mapsdown>")
+  ("| - > var var var" "<hookdownarrow>")
+  ("| - > var var var var" "<mapsup>")
+  ("| - > var var var var var" "<hookuparrow>")
+  ("| - - >" "<longmapsto>")
+  ("| - - > var" "<longhookrightarrow>")
+  ("| - - > var var" "<longmapsdown>")
+  ("| - - > var var var" "<longhookdownarrow>")
+  ("| - - > var var var var" "<longmapsup>")
+  ("| - - > var var var var var" "<longhookuparrow>")
   ("< - |" "<mapsfrom>")
   ("< - | var" "<hookleftarrow>")
   ("< - | var var" "<mapsup>")
@@ -970,17 +1034,33 @@
   ("- > - >" "<rightrightarrows>")
   ("- > - > var" "<downdownarrows>")
   ("- > - > var var" "<upuparrows>")
+  ("- > - > - >" "<threerightarrows>")
+  ("- > - > - > - >" "<fourrightarrows>")
   ("< - < -" "<leftleftarrows>")
   ("< - < - var" "<upuparrows>")
   ("< - < - var var" "<downdownarrows>")
+  ("< - < - < -" "<threeleftarrows>")
+  ("< - < - < - < -" "<fourleftarrows>")
   ("- > < -" "<rightleftarrows>")
   ("- > < - var" "<rightleftharpoons>")
+  ("- - > - >" "<longrightrightarrows>")
+  ("- - > - - >" "<longrightrightarrows>")
   ("- - > < -" "<longrightleftarrows>")
   ("- - > < - var" "<longrightleftharpoons>")
   ("- - > < - -" "<longrightleftarrows>")
   ("- - > < - - var" "<longrightleftharpoons>")
+  ("- - > - > - >" "<longthreerightarrows>")
+  ("- - > - - > - - >" "<longthreerightarrows>")
+  ("- - > - > - > - >" "<longfourrightarrows>")
+  ("- - > - - > - - > - - >" "<longfourrightarrows>")
   ("< - - > var var" "<leftrightarrows>")
   ("< - - > var var var" "<leftrightharpoons>")
+  ("< - - < -" "<longleftleftarrows>")
+  ("< - - < - -" "<longleftleftarrows>")
+  ("< - - < - < -" "<longthreeleftarrows>")
+  ("< - - < - - < - -" "<longthreeleftarrows>")
+  ("< - - < - < - < -" "<longfourleftarrows>")
+  ("< - - < - - < - - < - -" "<longfourleftarrows>")
   ("< - - - >" "<longleftrightarrows>")
   ("< - - - > var" "<longleftrightharpoons>")
   ("< - - - - >" "<longleftrightarrows>")
@@ -1028,6 +1108,7 @@
   ("< ~ > !" "<leftrightsquigarrowlim>")
   ("= !" "<equallim>")
   ("= = !" "<longequallim>")
+  ("= = var var !" "<longequivlim>")
   ("= > !" "<Rightarrowlim>")
   ("< = !" "<Leftarrowlim>")
   ("< = > !" "<Leftrightarrowlim>")
@@ -1056,7 +1137,21 @@
   ("= > ^" (make-long-arrow "<Rightarrow>"))
   ("< = ^" (make-long-arrow "<Leftarrow>"))
   ("< = > ^" (make-long-arrow "<Leftrightarrow>"))
+  ("= var var ^" (make-long-arrow "<equiv>"))
+  ("= var var > ^" (make-long-arrow "<Rrightarrow>"))
+  ("= > var var var ^" (make-long-arrow "<Rrightarrow>"))
+  ("< = var var var var ^" (make-long-arrow "<Lleftarrow>"))
+  ("< = var var var var > ^" (make-long-arrow "<LRleftrightarrow>"))
+  ("< = > var var ^" (make-long-arrow "<LRleftrightarrow>"))
   ("@ - ^" (make-long-arrow "<mapmulti>"))
+  ("- > - > ^" (make-long-arrow "<rightrightarrows>"))
+  ("< - < - ^" (make-long-arrow "<leftleftarrows>"))
+  ("- > < - ^" (make-long-arrow "<rightleftarrows>"))
+  ("< - - > var var ^" (make-long-arrow "<leftrightarrows>"))
+  ("- > - > - > ^" (make-long-arrow "<threerightarrows>"))
+  ("< - < - < - ^" (make-long-arrow "<threeleftarrows>"))
+  ("- > - > - > - > ^" (make-long-arrow "<fourrightarrows>"))
+  ("< - < - < - < - ^" (make-long-arrow "<fourleftarrows>"))
 
   ("- - ^" (make-long-arrow "<minus>"))
   ("- - > ^" (make-long-arrow "<rightarrow>"))
@@ -1088,7 +1183,17 @@
   ("= = > ^" (make-long-arrow "<Rightarrow>"))
   ("< = = ^" (make-long-arrow "<Leftarrow>"))
   ("< = = > ^" (make-long-arrow "<Leftrightarrow>"))
+  ("= = var var ^" (make-long-arrow "<equiv>"))
+  ("= = var var > ^" (make-long-arrow "<Rrightarrow>"))
+  ("= = > var var var ^" (make-long-arrow "<Rrightarrow>"))
+  ("< = = var var var ^" (make-long-arrow "<Lleftarrow>"))
+  ("< = = var var var > ^" (make-long-arrow "<LRleftrightarrow>"))
+  ("< = = > var var ^" (make-long-arrow "<LRleftrightarrow>"))
   ("@ - - ^" (make-long-arrow "<mapmulti>"))
+  ("- - > - > ^" (make-long-arrow "<rightrightarrows>"))
+  ("- - > - - > ^" (make-long-arrow "<rightrightarrows>"))
+  ("< - - < - ^" (make-long-arrow "<leftleftarrows>"))
+  ("< - - < - - ^" (make-long-arrow "<leftleftarrows>"))
 
   ("- _" (make-long-arrow* "<minus>"))
   ("- > _" (make-long-arrow* "<rightarrow>"))
@@ -1111,7 +1216,21 @@
   ("= > _" (make-long-arrow* "<Rightarrow>"))
   ("< = _" (make-long-arrow* "<Leftarrow>"))
   ("< = > _" (make-long-arrow* "<Leftrightarrow>"))
+  ("= var var _" (make-long-arrow* "<equiv>"))
+  ("= var var > _" (make-long-arrow* "<Rrightarrow>"))
+  ("= > var var var _" (make-long-arrow* "<Rrightarrow>"))
+  ("< = var var var var _" (make-long-arrow* "<Lleftarrow>"))
+  ("< = var var var var > _" (make-long-arrow* "<LRleftrightarrow>"))
+  ("< = > var var _" (make-long-arrow* "<LRleftrightarrow>"))
   ("@ - _" (make-long-arrow* "<mapmulti>"))
+  ("- > - > _" (make-long-arrow* "<rightrightarrows>"))
+  ("< - < - _" (make-long-arrow* "<leftleftarrows>"))
+  ("- > < - _" (make-long-arrow* "<rightleftarrows>"))
+  ("< - - > var var _" (make-long-arrow* "<leftrightarrows>"))
+  ("- > - > - > _" (make-long-arrow* "<threerightarrows>"))
+  ("< - < - < - _" (make-long-arrow* "<threeleftarrows>"))
+  ("- > - > - > - > _" (make-long-arrow* "<fourrightarrows>"))
+  ("< - < - < - < - _" (make-long-arrow* "<fourleftarrows>"))
 
   ("- - _" (make-long-arrow* "<minus>"))
   ("- - > _" (make-long-arrow* "<rightarrow>"))
@@ -1143,7 +1262,17 @@
   ("= = > _" (make-long-arrow* "<Rightarrow>"))
   ("< = = _" (make-long-arrow* "<Leftarrow>"))
   ("< = = > _" (make-long-arrow* "<Leftrightarrow>"))
+  ("= = var var _" (make-long-arrow* "<equiv>"))
+  ("= = var var > _" (make-long-arrow* "<Rrightarrow>"))
+  ("= = > var var var _" (make-long-arrow* "<Rrightarrow>"))
+  ("< = = var var var _" (make-long-arrow* "<Lleftarrow>"))
+  ("< = = var var var > _" (make-long-arrow* "<LRleftrightarrow>"))
+  ("< = = > var var _" (make-long-arrow* "<LRleftrightarrow>"))
   ("@ - - _" (make-long-arrow* "<mapmulti>"))
+  ("- - > - > _" (make-long-arrow* "<rightrightarrows>"))
+  ("- - > - - > _" (make-long-arrow* "<rightrightarrows>"))
+  ("< - - < - _" (make-long-arrow* "<leftleftarrows>"))
+  ("< - - < - - _" (make-long-arrow* "<leftleftarrows>"))
 
   ("@" "<circ>")
   ("@ /" "<varnothing>")
@@ -1301,14 +1430,17 @@
   ("% - . ." "<veebar><cdots><veebar>")
 
   ("| /" "<nmid>")
+  ("| / var" "<ndivides>")
   ("| var" "|")
   ("| var /" "<nmid>")
+  ("| var / var" "<ndivides>")
   ("| var var" (math-separator "|" 'default))
   ("| var var/" "<nmid>")
   ("| var var var" "<shortmid>")
   ("| var var var /" "<nshortmid>")
   ("| var var var var" "<barsuchthat>")
   ("| var var var var var" "<divides>")
+  ("| var var var var var /" "<ndivides>")
   ("| | var" "<||>")
   ("| | var /" "<nparallel>")
   ("| | var var" (math-separator "<||>" 'default))
@@ -1444,6 +1576,7 @@
   ("- +" "<mp>")
   ("+ - . ." "<pm><cdots><pm>")
   ("- + . ." "<mp><cdots><mp>")
+  ("+ + var" "<pplus>")
   ("@ =" "<circeq>")
   ("@ = var" "<triangleq>")
   ("= @" "<eqcirc>")
@@ -1502,6 +1635,23 @@
   ("* var var | var" "<btimes>")
   ("* var var | var var" "<ttimes>")
   ("! var" "<neg>")
+  
+  ("math:misc &" "&")
+  ("math:misc %" "%")
+  ("math:misc % var" "<permil>")
+  ("math:misc \\" "\\")
+  ("math:misc \\ var" "<setminus>")
+  ("math:misc \\ var var" "<smallsetminus>")
+  ("math:misc |" "|")
+  ("math:misc | /" "<nmid>")
+  ("math:misc | var" "<shortmid>")
+  ("math:misc | var /" "<nshortmid>")
+  ("math:misc | var var" "<divides>")
+  ("math:misc | var var /" "<ndivides>")
+  ("math:misc | |" "<||>")
+  ("math:misc | | /" "<nparallel>")
+  ("math:misc | | var" "<shortparallel>")
+  ("math:misc | | var /" "<nshortparallel>")
 
   ("math:greek a" "<alpha>")
   ("math:greek b" "<beta>")
@@ -1564,6 +1714,7 @@
   ("math:greek P" "<Pi>")
   ("math:greek R" "<Rho>")
   ("math:greek S" "<Sigma>")
+  ("math:greek S var" "<Backsigma>")
   ("math:greek C" "<Sigma>")
   ("math:greek T" "<Tau>")
   ("math:greek U" "<Upsilon>")
@@ -1572,25 +1723,120 @@
   ("math:greek Q" "<Chi>")
   ("math:greek W" "<Omega>")
   ("math:greek W var" "<Mho>")
-  ("math:greek V" "<Phi>")
+  ("math:greek V" "<Phi>"))
 
-  ("math:greek &" "&")
-  ("math:greek %" "%")
-  ("math:greek % var" "<permil>")
-  ("math:greek \\" "\\")
-  ("math:greek \\ var" "<setminus>")
-  ("math:greek \\ var var" "<smallsetminus>")
-  ("math:greek |" "|")
-  ("math:greek | /" "<nmid>")
-  ("math:greek | var" "<shortmid>")
-  ("math:greek | var /" "<nshortmid>")
-  ("math:greek | var var" "<divides>")
-  ("math:greek | var var /" "<ndivides>")
-  ("math:greek | |" "<||>")
-  ("math:greek | | /" "<nparallel>")
-  ("math:greek | | var" "<shortparallel>")
-  ("math:greek | | var /" "<nshortparallel>")
+(kbd-map
+  (:mode in-math-not-hybrid?)
+  ("a var" "<alpha>")
+  ("b var" "<beta>")
+  ("b var var" "<flat>")
+  ("c var" "<varsigma>")
+  ("c var var" "<sigma>")
+  ("d var" "<delta>")
+  ("d var var" "<mathd>")
+  ("d var var var" "<partial>")
+  ("e var" "<varepsilon>")
+  ("e var var" "<mathe>")
+  ("e var var var" "<epsilon>")
+  ("e var var var var" "<backepsilon>")
+  ("f var" "<varphi>")
+  ("f var var" "<phi>")
+  ("g var" "<gamma>")
+  ("g var var" "<matheuler>")
+  ("h var" "<eta>")
+  ("h var var" "<hbar>")
+  ("i var" "<iota>")
+  ("i var var" "<mathi>")
+  ("i var var var" "<imath>")
+  ("j var" "<theta>")
+  ("j var var" "<jmath>")
+  ("j var var var" "<vartheta>")
+  ("k var" "<kappa>")
+  ("k var var" "<varkappa>")
+  ("l var" "<lambda>")
+  ("l var var" "<mathlambda>")
+  ("l var var var" "<ell>")
+  ("m var" "<mu>")
+  ("n var" "<nu>")
+  ("o var" "<omicron>")
+  ("p var" "<pi>")
+  ("p var var" "<mathpi>")
+  ("p var var var" "<varpi>")
+  ("q var" "<chi>")
+  ("r var" "<rho>")
+  ("r var var" "<varrho>")
+  ("s var" "<sigma>")
+  ("s var var" "<varsigma>")
+  ("t var" "<tau>")
+  ("u var" "<upsilon>")
+  ("v var" "<phi>")
+  ("v var var" "<varphi>")
+  ("w var" "<omega>")
+  ("w var var" "<mho>")
+  ("x var" "<xi>")
+  ("y var" "<psi>")
+  ("z var" "<zeta>")
 
+  ("A var" "<Alpha>")
+  ("A var var" "<forall>")
+  ("A var var var" "<aleph>")
+  ("B var" "<Beta>")
+  ("B var var" "<bot>")
+  ("B var var var" "<beth>")
+  ("C var" "<Sigma>")
+  ("C var var" "<mathcatalan>")
+  ("C var var var" "<complement>")
+  ("D var" "<Delta>")
+  ("D var var" "<mathD>")
+  ("D var var var" "<daleth>")
+  ("D var var var var" "<mathLaplace>")
+  ("E var" "<Epsilon>")
+  ("E var var" "<exists>")
+  ("E var var var" "<Exists>")
+  ("E var var var var" "<Backepsilon>")
+  ("F var" "<Phi>")
+  ("G var" "<Gamma>")
+  ("G var var" "<gimel>")
+  ("G var var var" "<mathGamma>")
+  ("G var var var var" "<digamma>")
+  ("H var" "<Eta>")
+  ("I var" "<Iota>")
+  ("I var var" "<Im>")
+  ("J var" "<Theta>")
+  ("K var" "<Kappa>")
+  ("L var" "<Lambda>")
+  ("M var" "<Mu>")
+  ("N var" "<Nu>")
+  ("O var" "<Omicron>")
+  ("P var" "<Pi>")
+  ("P var var" "<wp>")
+  ("Q var" "<Chi>")
+  ("R var" "<Rho>")
+  ("R var var" "<Re>")
+  ("S var" "<Sigma>")
+  ("S var var" "<Backsigma>")
+  ("T var" "<Tau>")
+  ("T var var" "<top>")
+  ("T var var var" "<bot>")
+  ("T var var var var" "<leftthreetimes>")
+  ("T var var var var var" "<rightthreetimes>")
+  ("T var var var var var var" "<lefttimesthree>")
+  ("T var var var var var var var" "<righttimesthree>")
+  ("U var" "<Upsilon>")
+  ("V var" "<Phi>")
+  ("V var var" "<nabla>")
+  ("W var" "<Omega>")
+  ("W var var" "<Mho>")
+  ("X var" "<Xi>")
+  ("Y var" "<Psi>")
+  ("Y var var" "<Ydown>")
+  ("Y var var var" "<Yup>")
+  ("Y var var var var" "<Yleft>")
+  ("Y var var var var var" "<Yright>")
+  ("Z var" "<Zeta>"))
+
+(kbd-map
+  (:mode in-math?)
   ("math:bold 0" "<b-0>")
   ("math:bold 1" "<b-1>")
   ("math:bold 2" "<b-2>")
@@ -1659,59 +1905,6 @@
   ("math:bold ~" "<thicksim>")
   ("math:bold ~ ~" "<thickapprox>")
 
-  ("math:bold:up a" "<b-up-a>")
-  ("math:bold:up b" "<b-up-b>")
-  ("math:bold:up c" "<b-up-c>")
-  ("math:bold:up d" "<b-up-d>")
-  ("math:bold:up e" "<b-up-e>")
-  ("math:bold:up f" "<b-up-f>")
-  ("math:bold:up g" "<b-up-g>")
-  ("math:bold:up h" "<b-up-h>")
-  ("math:bold:up i" "<b-up-i>")
-  ("math:bold:up j" "<b-up-j>")
-  ("math:bold:up k" "<b-up-k>")
-  ("math:bold:up l" "<b-up-l>")
-  ("math:bold:up m" "<b-up-m>")
-  ("math:bold:up n" "<b-up-n>")
-  ("math:bold:up o" "<b-up-o>")
-  ("math:bold:up p" "<b-up-p>")
-  ("math:bold:up q" "<b-up-q>")
-  ("math:bold:up r" "<b-up-r>")
-  ("math:bold:up s" "<b-up-s>")
-  ("math:bold:up t" "<b-up-t>")
-  ("math:bold:up u" "<b-up-u>")
-  ("math:bold:up v" "<b-up-v>")
-  ("math:bold:up w" "<b-up-w>")
-  ("math:bold:up x" "<b-up-x>")
-  ("math:bold:up y" "<b-up-y>")
-  ("math:bold:up z" "<b-up-z>")
-  ("math:bold:up A" "<b-up-A>")
-  ("math:bold:up B" "<b-up-B>")
-  ("math:bold:up C" "<b-up-C>")
-  ("math:bold:up D" "<b-up-D>")
-  ("math:bold:up E" "<b-up-E>")
-  ("math:bold:up F" "<b-up-F>")
-  ("math:bold:up G" "<b-up-G>")
-  ("math:bold:up H" "<b-up-H>")
-  ("math:bold:up I" "<b-up-I>")
-  ("math:bold:up J" "<b-up-J>")
-  ("math:bold:up K" "<b-up-K>")
-  ("math:bold:up L" "<b-up-L>")
-  ("math:bold:up M" "<b-up-M>")
-  ("math:bold:up N" "<b-up-N>")
-  ("math:bold:up O" "<b-up-O>")
-  ("math:bold:up P" "<b-up-P>")
-  ("math:bold:up Q" "<b-up-Q>")
-  ("math:bold:up R" "<b-up-R>")
-  ("math:bold:up S" "<b-up-S>")
-  ("math:bold:up T" "<b-up-T>")
-  ("math:bold:up U" "<b-up-U>")
-  ("math:bold:up V" "<b-up-V>")
-  ("math:bold:up W" "<b-up-W>")
-  ("math:bold:up X" "<b-up-X>")
-  ("math:bold:up Y" "<b-up-Y>")
-  ("math:bold:up Z" "<b-up-Z>")
-
   ("math:bold:greek a" "<b-alpha>")
   ("math:bold:greek b" "<b-beta>")
   ("math:bold:greek g" "<b-gamma>")
@@ -1768,6 +1961,7 @@
   ("math:bold:greek P" "<b-Pi>")
   ("math:bold:greek R" "<b-Rho>")
   ("math:bold:greek S" "<b-Sigma>")
+  ("math:bold:greek S var" "<b-Backsigma>")
   ("math:bold:greek C" "<b-Sigma>")
   ("math:bold:greek T" "<b-Tau>")
   ("math:bold:greek U" "<b-Upsilon>")
@@ -1778,6 +1972,314 @@
   ("math:bold:greek W var" "<b-Mho>")
   ("math:bold:greek V" "<b-Phi>")
 
+  ("math:bold a var" "<b-alpha>")
+  ("math:bold b var" "<b-beta>")
+  ("math:bold c var" "<b-varsigma>")
+  ("math:bold c var var" "<b-sigma>")
+  ("math:bold d var" "<b-delta>")
+  ("math:bold e var" "<b-varepsilon>")
+  ("math:bold e var var" "<b-epsilon>")
+  ("math:bold e var var var" "<b-backepsilon>")
+  ("math:bold f var" "<b-varphi>")
+  ("math:bold f var var" "<b-phi>")
+  ("math:bold g var" "<b-gamma>")
+  ("math:bold h var" "<b-eta>")
+  ("math:bold i var" "<b-iota>")
+  ("math:bold i var var" "<b-imath>")
+  ("math:bold j var" "<b-theta>")
+  ("math:bold j var var" "<b-jmath>")
+  ("math:bold j var var var" "<b-vartheta>")
+  ("math:bold k var" "<b-kappa>")
+  ("math:bold k var var" "<b-varkappa>")
+  ("math:bold l var" "<b-lambda>")
+  ("math:bold l var var" "<b-ell>")
+  ("math:bold m var" "<b-mu>")
+  ("math:bold n var" "<b-nu>")
+  ("math:bold o var" "<b-omicron>")
+  ("math:bold p var" "<b-pi>")
+  ("math:bold p var var" "<b-varpi>")
+  ("math:bold q var" "<b-chi>")
+  ("math:bold r var" "<b-rho>")
+  ("math:bold r var var" "<b-varrho>")
+  ("math:bold s var" "<b-sigma>")
+  ("math:bold s var var" "<b-varsigma>")
+  ("math:bold t var" "<b-tau>")
+  ("math:bold u var" "<b-upsilon>")
+  ("math:bold v var" "<b-phi>")
+  ("math:bold v var var" "<b-varphi>")
+  ("math:bold w var" "<b-omega>")
+  ("math:bold w var var" "<b-mho>")
+  ("math:bold x var" "<b-xi>")
+  ("math:bold y var" "<b-psi>")
+  ("math:bold z var" "<b-zeta>")
+  ("math:bold A var" "<b-Alpha>")
+  ("math:bold B var" "<b-Beta>")
+  ("math:bold C var" "<b-Sigma>")
+  ("math:bold D var" "<b-Delta>")
+  ("math:bold E var" "<b-Epsilon>")
+  ("math:bold E var var" "<b-Backepsilon>")
+  ("math:bold F var" "<b-Phi>")
+  ("math:bold G var" "<b-Gamma>")
+  ("math:bold H var" "<b-Eta>")
+  ("math:bold I var" "<b-Iota>")
+  ("math:bold J var" "<b-Theta>")
+  ("math:bold K var" "<b-Kappa>")
+  ("math:bold L var" "<b-Lambda>")
+  ("math:bold M var" "<b-Mu>")
+  ("math:bold N var" "<b-Nu>")
+  ("math:bold O var" "<b-Omicron>")
+  ("math:bold P var" "<b-Pi>")
+  ("math:bold Q var" "<b-Chi>")
+  ("math:bold R var" "<b-Rho>")
+  ("math:bold S var" "<b-Sigma>")
+  ("math:bold S var var" "<b-Backsigma>")
+  ("math:bold T var" "<b-Tau>")
+  ("math:bold U var" "<b-Upsilon>")
+  ("math:bold V var" "<b-Phi>")
+  ("math:bold W var" "<b-Omega>")
+  ("math:bold W var var" "<b-Mho>")
+  ("math:bold X var" "<b-Xi>")
+  ("math:bold Y var" "<b-Psi>")
+  ("math:bold Z var" "<b-Zeta>")
+
+  ("math:up a" "<up-a>")
+  ("math:up b" "<up-b>")
+  ("math:up c" "<up-c>")
+  ("math:up d" "<up-d>")
+  ("math:up e" "<up-e>")
+  ("math:up f" "<up-f>")
+  ("math:up g" "<up-g>")
+  ("math:up h" "<up-h>")
+  ("math:up i" "<up-i>")
+  ("math:up j" "<up-j>")
+  ("math:up k" "<up-k>")
+  ("math:up l" "<up-l>")
+  ("math:up m" "<up-m>")
+  ("math:up n" "<up-n>")
+  ("math:up o" "<up-o>")
+  ("math:up p" "<up-p>")
+  ("math:up q" "<up-q>")
+  ("math:up r" "<up-r>")
+  ("math:up s" "<up-s>")
+  ("math:up t" "<up-t>")
+  ("math:up u" "<up-u>")
+  ("math:up v" "<up-v>")
+  ("math:up w" "<up-w>")
+  ("math:up x" "<up-x>")
+  ("math:up y" "<up-y>")
+  ("math:up z" "<up-z>")
+  ("math:up A" "<up-A>")
+  ("math:up B" "<up-B>")
+  ("math:up C" "<up-C>")
+  ("math:up D" "<up-D>")
+  ("math:up E" "<up-E>")
+  ("math:up F" "<up-F>")
+  ("math:up G" "<up-G>")
+  ("math:up H" "<up-H>")
+  ("math:up I" "<up-I>")
+  ("math:up J" "<up-J>")
+  ("math:up K" "<up-K>")
+  ("math:up L" "<up-L>")
+  ("math:up M" "<up-M>")
+  ("math:up N" "<up-N>")
+  ("math:up O" "<up-O>")
+  ("math:up P" "<up-P>")
+  ("math:up Q" "<up-Q>")
+  ("math:up R" "<up-R>")
+  ("math:up S" "<up-S>")
+  ("math:up T" "<up-T>")
+  ("math:up U" "<up-U>")
+  ("math:up V" "<up-V>")
+  ("math:up W" "<up-W>")
+  ("math:up X" "<up-X>")
+  ("math:up Y" "<up-Y>")
+  ("math:up Z" "<up-Z>")
+
+  ("math:up a var" "<up-alpha>")
+  ("math:up b var" "<up-beta>")
+  ("math:up c var" "<up-varsigma>")
+  ("math:up c var var" "<up-sigma>")
+  ("math:up d var" "<up-delta>")
+  ("math:up e var" "<up-varepsilon>")
+  ("math:up e var var" "<up-epsilon>")
+  ("math:up f var" "<up-varphi>")
+  ("math:up f var var" "<up-phi>")
+  ("math:up g var" "<up-gamma>")
+  ("math:up h var" "<up-eta>")
+  ("math:up i var" "<up-iota>")
+  ("math:up i var var" "<up-imath>")
+  ("math:up j var" "<up-theta>")
+  ("math:up j var var" "<up-jmath>")
+  ("math:up j var var var" "<up-vartheta>")
+  ("math:up k var" "<up-kappa>")
+  ("math:up k var var" "<up-varkappa>")
+  ("math:up l var" "<up-lambda>")
+  ("math:up l var var" "<up-ell>")
+  ("math:up m var" "<up-mu>")
+  ("math:up n var" "<up-nu>")
+  ("math:up o var" "<up-omicron>")
+  ("math:up p var" "<up-pi>")
+  ("math:up p var var" "<up-varpi>")
+  ("math:up q var" "<up-chi>")
+  ("math:up r var" "<up-rho>")
+  ("math:up r var var" "<up-varrho>")
+  ("math:up s var" "<up-sigma>")
+  ("math:up s var var" "<up-varsigma>")
+  ("math:up t var" "<up-tau>")
+  ("math:up u var" "<up-upsilon>")
+  ("math:up v var" "<up-phi>")
+  ("math:up v var var" "<up-varphi>")
+  ("math:up w var" "<up-omega>")
+  ("math:up x var" "<up-xi>")
+  ("math:up y var" "<up-psi>")
+  ("math:up z var" "<up-zeta>")
+  ;;("math:up A var" "<up-Alpha>")
+  ;;("math:up B var" "<up-Beta>")
+  ;;("math:up C var" "<up-Sigma>")
+  ;;("math:up D var" "<up-Delta>")
+  ;;("math:up E var" "<up-Epsilon>")
+  ;;("math:up F var" "<up-Phi>")
+  ;;("math:up G var" "<up-Gamma>")
+  ;;("math:up H var" "<up-Eta>")
+  ;;("math:up I var" "<up-Iota>")
+  ;;("math:up J var" "<up-Theta>")
+  ;;("math:up K var" "<up-Kappa>")
+  ;;("math:up L var" "<up-Lambda>")
+  ;;("math:up M var" "<up-Mu>")
+  ;;("math:up N var" "<up-Nu>")
+  ;;("math:up O var" "<up-Omicron>")
+  ;;("math:up P var" "<up-Pi>")
+  ;;("math:up Q var" "<up-Chi>")
+  ;;("math:up R var" "<up-Rho>")
+  ;;("math:up S var" "<up-Sigma>")
+  ;;("math:up T var" "<up-Tau>")
+  ;;("math:up U var" "<up-Upsilon>")
+  ;;("math:up V var" "<up-Phi>")
+  ;;("math:up W var" "<up-Omega>")
+  ;;("math:up W var var" "<up-Mho>")
+  ;;("math:up X var" "<up-Xi>")
+  ;;("math:up Y var" "<up-Psi>")
+  ;;("math:up Z var" "<up-Zeta>")
+
+  ("math:bold:up a" "<b-up-a>")
+  ("math:bold:up b" "<b-up-b>")
+  ("math:bold:up c" "<b-up-c>")
+  ("math:bold:up d" "<b-up-d>")
+  ("math:bold:up e" "<b-up-e>")
+  ("math:bold:up f" "<b-up-f>")
+  ("math:bold:up g" "<b-up-g>")
+  ("math:bold:up h" "<b-up-h>")
+  ("math:bold:up i" "<b-up-i>")
+  ("math:bold:up j" "<b-up-j>")
+  ("math:bold:up k" "<b-up-k>")
+  ("math:bold:up l" "<b-up-l>")
+  ("math:bold:up m" "<b-up-m>")
+  ("math:bold:up n" "<b-up-n>")
+  ("math:bold:up o" "<b-up-o>")
+  ("math:bold:up p" "<b-up-p>")
+  ("math:bold:up q" "<b-up-q>")
+  ("math:bold:up r" "<b-up-r>")
+  ("math:bold:up s" "<b-up-s>")
+  ("math:bold:up t" "<b-up-t>")
+  ("math:bold:up u" "<b-up-u>")
+  ("math:bold:up v" "<b-up-v>")
+  ("math:bold:up w" "<b-up-w>")
+  ("math:bold:up x" "<b-up-x>")
+  ("math:bold:up y" "<b-up-y>")
+  ("math:bold:up z" "<b-up-z>")
+  ("math:bold:up A" "<b-up-A>")
+  ("math:bold:up B" "<b-up-B>")
+  ("math:bold:up C" "<b-up-C>")
+  ("math:bold:up D" "<b-up-D>")
+  ("math:bold:up E" "<b-up-E>")
+  ("math:bold:up F" "<b-up-F>")
+  ("math:bold:up G" "<b-up-G>")
+  ("math:bold:up H" "<b-up-H>")
+  ("math:bold:up I" "<b-up-I>")
+  ("math:bold:up J" "<b-up-J>")
+  ("math:bold:up K" "<b-up-K>")
+  ("math:bold:up L" "<b-up-L>")
+  ("math:bold:up M" "<b-up-M>")
+  ("math:bold:up N" "<b-up-N>")
+  ("math:bold:up O" "<b-up-O>")
+  ("math:bold:up P" "<b-up-P>")
+  ("math:bold:up Q" "<b-up-Q>")
+  ("math:bold:up R" "<b-up-R>")
+  ("math:bold:up S" "<b-up-S>")
+  ("math:bold:up T" "<b-up-T>")
+  ("math:bold:up U" "<b-up-U>")
+  ("math:bold:up V" "<b-up-V>")
+  ("math:bold:up W" "<b-up-W>")
+  ("math:bold:up X" "<b-up-X>")
+  ("math:bold:up Y" "<b-up-Y>")
+  ("math:bold:up Z" "<b-up-Z>")
+
+  ("math:bold:up a var" "<b-up-alpha>")
+  ("math:bold:up b var" "<b-up-beta>")
+  ("math:bold:up c var" "<b-up-varsigma>")
+  ("math:bold:up c var var" "<b-up-sigma>")
+  ("math:bold:up d var" "<b-up-delta>")
+  ("math:bold:up e var" "<b-up-varepsilon>")
+  ("math:bold:up e var var" "<b-up-epsilon>")
+  ("math:bold:up f var" "<b-up-varphi>")
+  ("math:bold:up f var var" "<b-up-phi>")
+  ("math:bold:up g var" "<b-up-gamma>")
+  ("math:bold:up h var" "<b-up-eta>")
+  ("math:bold:up i var" "<b-up-iota>")
+  ("math:bold:up i var var" "<b-up-imath>")
+  ("math:bold:up j var" "<b-up-theta>")
+  ("math:bold:up j var var" "<b-up-jmath>")
+  ("math:bold:up j var var var" "<b-up-vartheta>")
+  ("math:bold:up k var" "<b-up-kappa>")
+  ("math:bold:up k var var" "<b-up-varkappa>")
+  ("math:bold:up l var" "<b-up-lambda>")
+  ("math:bold:up l var var" "<b-up-ell>")
+  ("math:bold:up m var" "<b-up-mu>")
+  ("math:bold:up n var" "<b-up-nu>")
+  ("math:bold:up o var" "<b-up-omicron>")
+  ("math:bold:up p var" "<b-up-pi>")
+  ("math:bold:up p var var" "<b-up-varpi>")
+  ("math:bold:up q var" "<b-up-chi>")
+  ("math:bold:up r var" "<b-up-rho>")
+  ("math:bold:up r var var" "<b-up-varrho>")
+  ("math:bold:up s var" "<b-up-sigma>")
+  ("math:bold:up s var var" "<b-up-varsigma>")
+  ("math:bold:up t var" "<b-up-tau>")
+  ("math:bold:up u var" "<b-up-upsilon>")
+  ("math:bold:up v var" "<b-up-phi>")
+  ("math:bold:up v var var" "<b-up-varphi>")
+  ("math:bold:up w var" "<b-up-omega>")
+  ("math:bold:up x var" "<b-up-xi>")
+  ("math:bold:up y var" "<b-up-psi>")
+  ("math:bold:up z var" "<b-up-zeta>")
+  ;;("math:bold:up A var" "<b-up-Alpha>")
+  ;;("math:bold:up B var" "<b-up-Beta>")
+  ;;("math:bold:up C var" "<b-up-Sigma>")
+  ;;("math:bold:up D var" "<b-up-Delta>")
+  ;;("math:bold:up E var" "<b-up-Epsilon>")
+  ;;("math:bold:up F var" "<b-up-Phi>")
+  ;;("math:bold:up G var" "<b-up-Gamma>")
+  ;;("math:bold:up H var" "<b-up-Eta>")
+  ;;("math:bold:up I var" "<b-up-Iota>")
+  ;;("math:bold:up J var" "<b-up-Theta>")
+  ;;("math:bold:up K var" "<b-up-Kappa>")
+  ;;("math:bold:up L var" "<b-up-Lambda>")
+  ;;("math:bold:up M var" "<b-up-Mu>")
+  ;;("math:bold:up N var" "<b-up-Nu>")
+  ;;("math:bold:up O var" "<b-up-Omicron>")
+  ;;("math:bold:up P var" "<b-up-Pi>")
+  ;;("math:bold:up Q var" "<b-up-Chi>")
+  ;;("math:bold:up R var" "<b-up-Rho>")
+  ;;("math:bold:up S var" "<b-up-Sigma>")
+  ;;("math:bold:up T var" "<b-up-Tau>")
+  ;;("math:bold:up U var" "<b-up-Upsilon>")
+  ;;("math:bold:up V var" "<b-up-Phi>")
+  ;;("math:bold:up W var" "<b-up-Omega>")
+  ;;("math:bold:up W var var" "<b-up-Mho>")
+  ;;("math:bold:up X var" "<b-up-Xi>")
+  ;;("math:bold:up Y var" "<b-up-Psi>")
+  ;;("math:bold:up Z var" "<b-up-Zeta>")
+
   ("math:cal a" "<cal-a>")
   ("math:cal b" "<cal-b>")
   ("math:cal c" "<cal-c>")
@@ -2000,8 +2502,10 @@
   ("math:bbb W" "<bbb-W>")
   ("math:bbb X" "<bbb-X>")
   ("math:bbb Y" "<bbb-Y>")
-  ("math:bbb Z" "<bbb-Z>")
+  ("math:bbb Z" "<bbb-Z>"))
 
+(kbd-map
+  (:mode in-math-not-hybrid?)
   ("A A" "<bbb-A>")
   ("B B" "<bbb-B>")
   ("C C" "<bbb-C>")
@@ -2039,115 +2543,6 @@
   ("R E" "<Re>")
   ("I M" "<Im>"))
 
-(kbd-map
-  (:mode in-math-not-hybrid?)
-  ("a var" "<alpha>")
-  ("b var" "<beta>")
-  ("b var var" "<flat>")
-  ("s var var" "<varsigma>")
-  ("c var" "<varsigma>")
-  ("d var" "<delta>")
-  ("d var var" "<mathd>")
-  ("d var var var" "<partial>")
-  ("e var" "<varepsilon>")
-  ("e var var" "<mathe>")
-  ("e var var var" "<epsilon>")
-  ("e var var var var" "<backepsilon>")
-  ("v var var" "<varphi>")
-  ("f var" "<varphi>")
-  ("g var" "<gamma>")
-  ("g var var" "<matheuler>")
-  ("h var" "<eta>")
-  ("h var var" "<hbar>")
-  ("i var" "<iota>")
-  ("i var var" "<mathi>")
-  ("i var var var" "<imath>")
-  ("j var" "<theta>")
-  ("j var var" "<jmath>")
-  ("j var var var" "<vartheta>")
-  ("k var" "<kappa>")
-  ("k var var" "<varkappa>")
-  ("l var" "<lambda>")
-  ("l var var" "<mathlambda>")
-  ("l var var var" "<ell>")
-  ("m var" "<mu>")
-  ("n var" "<nu>")
-  ("o var" "<omicron>")
-  ("p var" "<pi>")
-  ("p var var" "<mathpi>")
-  ("p var var var" "<varpi>")
-  ("q var" "<chi>")
-  ("r var" "<rho>")
-  ("r var var" "<varrho>")
-  ("c var var" "<sigma>")
-  ("s var" "<sigma>")
-  ("t var" "<tau>")
-  ("u var" "<upsilon>")
-  ("f var var" "<phi>")
-  ("v var" "<phi>")
-  ("w var" "<omega>")
-  ("w var var" "<mho>")
-  ("x var" "<xi>")
-  ("y var" "<psi>")
-  ("z var" "<zeta>")
-
-  ("A var" "<Alpha>")
-  ("A var var" "<forall>")
-  ("A var var var" "<aleph>")
-  ("B var" "<Beta>")
-  ("B var var" "<bot>")
-  ("B var var var" "<beth>")
-  ("C var" "<Sigma>")
-  ("C var var" "<mathcatalan>")
-  ("C var var var" "<complement>")
-  ("D var" "<Delta>")
-  ("D var var" "<mathD>")
-  ("D var var var" "<daleth>")
-  ("D var var var var" "<mathLaplace>")
-  ("E var" "<Epsilon>")
-  ("E var var" "<exists>")
-  ("E var var var" "<Exists>")
-  ("E var var var var" "<Backepsilon>")
-  ("F var" "<Phi>")
-  ("G var" "<Gamma>")
-  ("G var var" "<gimel>")
-  ("G var var var" "<mathGamma>")
-  ("G var var var var" "<digamma>")
-  ("H var" "<Eta>")
-  ("I var" "<Iota>")
-  ("I var var" "<Im>")
-  ("J var" "<Theta>")
-  ("K var" "<Kappa>")
-  ("L var" "<Lambda>")
-  ("M var" "<Mu>")
-  ("N var" "<Nu>")
-  ("O var" "<Omicron>")
-  ("P var" "<Pi>")
-  ("P var var" "<wp>")
-  ("Q var" "<Chi>")
-  ("R var" "<Rho>")
-  ("R var var" "<Re>")
-  ("S var" "<Sigma>")
-  ("T var" "<Tau>")
-  ("T var var" "<top>")
-  ("T var var var" "<bot>")
-  ("T var var var var" "<leftthreetimes>")
-  ("T var var var var var" "<rightthreetimes>")
-  ("T var var var var var var" "<lefttimesthree>")
-  ("T var var var var var var var" "<righttimesthree>")
-  ("U var" "<Upsilon>")
-  ("V var" "<Phi>")
-  ("V var var" "<nabla>")
-  ("W var" "<Omega>")
-  ("W var var" "<Mho>")
-  ("X var" "<Xi>")
-  ("Y var" "<Psi>")
-  ("Y var var" "<Ydown>")
-  ("Y var var var" "<Yup>")
-  ("Y var var var var" "<Yleft>")
-  ("Y var var var var var" "<Yright>")
-  ("Z var" "<Zeta>"))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Hacks to work around the problem that MacOS reserves Alt-based shortcuts
 ;; for its own personal use
@@ -2162,3 +2557,112 @@
     ("table N M" (make 'bmatrix))
     ("table N d" (make 'det))
     ("table N s" (make 'stack))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Textual operators
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(kbd-map
+  (:mode in-math-english?)
+  ("a n d" (make 'infix-and))
+  ("a n d space" (make 'infix-and))
+  ("space a" " a")
+  ("space a n" " an")
+  ("space a n d" (make 'infix-and))
+  ("space a n d space" (make 'infix-and))
+  ("a n d var" "and")
+  ("o r" (make 'infix-or))
+  ("o r space" (make 'infix-or))
+  ("space o" " o")
+  ("space o r" (make 'infix-or))
+  ("space o r space" (make 'infix-or))
+  ("o r var" "or")
+  ("o r d" "ord")
+  ("i f f" (make 'infix-iff))
+  ("i f f space" (make 'infix-iff))
+  ("space i" " i")
+  ("space i f" " if")
+  ("space i f f" (make 'infix-iff))
+  ("space i f f space" (make 'infix-iff))
+  ("i f f var" "iff")
+  ("f o r space a l l" (make 'prefix-for-all))
+  ("f o r space a l l space" (make 'prefix-for-all)))
+
+(kbd-map
+  (:mode in-math-dutch?)
+  ("e n" (make 'infix-and))
+  ("e n space" (make 'infix-and))
+  ("space e" " e")
+  ("space e n" (make 'infix-and))
+  ("space e n space" (make 'infix-and))
+  ("e n var" "en")
+  ("o f" (make 'infix-or))
+  ("o f space" (make 'infix-or))
+  ("space o" " o")
+  ("space o f" (make 'infix-or))
+  ("space o f space" (make 'infix-or))
+  ("o f var" "of")
+  ("d e s d a" (make 'infix-iff))
+  ("d e s d a space" (make 'infix-iff))
+  ("space d" " d")
+  ("space d e" " de")
+  ("space d e s" " des")
+  ("space d e s d" " desd")
+  ("space d e s d a" (make 'infix-iff))
+  ("space d e s d a space" (make 'infix-iff))
+  ("d e s d a var" "desda")
+  ("v o o r space a l l e" (make 'prefix-for-all))
+  ("v o o r space a l l e space" (make 'prefix-for-all)))
+
+(kbd-map
+  (:mode in-math-french?)
+  ("e t" (make 'infix-and))
+  ("e t space" (make 'infix-and))
+  ("space e" " e")
+  ("space e t" (make 'infix-and))
+  ("space e t space" (make 'infix-and))
+  ("e t var" "et")
+  ("o u" (make 'infix-or))
+  ("o u space" (make 'infix-or))
+  ("space o" " o")
+  ("space o u" (make 'infix-or))
+  ("space o u space" (make 'infix-or))
+  ("o u var" "ou")
+  ("s s i" (make 'infix-iff))
+  ("s s i space" (make 'infix-iff))
+  ("space s" " s")
+  ("space s s" " ss")
+  ("space s s i" (make 'infix-iff))
+  ("space s s i space" (make 'infix-iff))
+  ("s s i var" "ssi")
+  ("p o u r space t o u t" (make 'prefix-for-all))
+  ("p o u r space t o u t space" (make 'prefix-for-all)))
+
+(kbd-map
+  (:mode in-math-german?)
+  ("u n d" (make 'infix-and))
+  ("u n d space" (make 'infix-and))
+  ("space u" " u")
+  ("space u n" " un")
+  ("space u n d" (make 'infix-and))
+  ("space u n d space" (make 'infix-and))
+  ("u n d var" "und")
+  ("o d e r" (make 'infix-or))
+  ("o d e r space" (make 'infix-or))
+  ("space o" " o")
+  ("space o d" " od")
+  ("space o d e" " ode")
+  ("space o d e r" (make 'infix-or))
+  ("space o d e r space" (make 'infix-or))
+  ("o d e r var" "oder")
+  ("g d w" (make 'infix-iff))
+  ("g d w space" (make 'infix-iff))
+  ("space g" " g")
+  ("space g d" " gd")
+  ("space g d w" (make 'infix-iff))
+  ("space g d w space" (make 'infix-iff))
+  ("g d w var" "gdw")
+  ("f u r space a l l e" (make 'prefix-for-all))
+  ("f u r space a l l e space" (make 'prefix-for-all))
+  ("f u e r space a l l e" (make 'prefix-for-all))
+  ("f u e r space a l l e space" (make 'prefix-for-all)))
